@@ -2,6 +2,7 @@
 """Classes and functions for finding and reading files."""
 import datetime
 from glob import glob
+import json
 import os
 from typing import Any
 from typing import Dict
@@ -14,7 +15,6 @@ from typing import Union
 from uuid import UUID
 
 import h5py
-from immutabledict import immutabledict
 from nptyping import NDArray
 import numpy as np
 from semver import VersionInfo
@@ -24,6 +24,7 @@ from .constants import CUSTOMER_ACCOUNT_ID_UUID
 from .constants import DATETIME_STR_FORMAT
 from .constants import FILE_FORMAT_VERSION_METADATA_KEY
 from .constants import IS_FILE_ORIGINAL_UNTRIMMED_UUID
+from .constants import MAGNETOMETER_CONFIGURATION_UUID
 from .constants import MANTARRAY_SERIAL_NUMBER_UUID
 from .constants import MICROSECONDS_PER_CENTIMILLISECOND
 from .constants import MIN_SUPPORTED_FILE_VERSION
@@ -31,6 +32,8 @@ from .constants import PLATE_BARCODE_UUID
 from .constants import REF_SAMPLING_PERIOD_UUID
 from .constants import REFERENCE_SENSOR_READINGS
 from .constants import START_RECORDING_TIME_INDEX_UUID
+from .constants import TIME_INDICES
+from .constants import TIME_OFFSETS
 from .constants import TISSUE_SAMPLING_PERIOD_UUID
 from .constants import TISSUE_SENSOR_READINGS
 from .constants import TRIMMED_TIME_FROM_ORIGINAL_START_UUID
@@ -41,7 +44,9 @@ from .constants import UTC_FIRST_REF_DATA_POINT_UUID
 from .constants import UTC_FIRST_TISSUE_DATA_POINT_UUID
 from .constants import WELL_INDEX_UUID
 from .constants import WELL_NAME_UUID
+from .exceptions import AxisDataForSensorNotInFileError
 from .exceptions import FileAttributeNotFoundError
+from .exceptions import SensorDataNotInFileError
 from .exceptions import UnsupportedMantarrayFileVersionError
 from .exceptions import WellRecordingsNotFromSameSessionError
 
@@ -64,7 +69,7 @@ def get_unique_files_from_directory(directory: str) -> List[str]:
     Returns:
         A list of the file paths for all the h5 files in the directory.
     """
-    unique_files: List[str] = []
+    unique_files: List[str] = list()
 
     for path, _, files in os.walk(directory):
         for name in files:
@@ -72,13 +77,13 @@ def get_unique_files_from_directory(directory: str) -> List[str]:
                 continue
 
             file = os.path.join(path, name)
-            well = WellFile(file)
+            well = BaseWellFile(file)
             index = well.get_well_index()
             barcode = well.get_plate_barcode()
             start_time = well.get_begin_recording()
 
             for item in unique_files:
-                new_well = WellFile(item)
+                new_well = BaseWellFile(item)
                 if (
                     new_well.get_well_index() == index
                     and new_well.get_plate_barcode() == barcode
@@ -107,7 +112,7 @@ def get_specified_files(
     plate_recording_list: List[str] = []
 
     for file in unique_files:
-        well = WellFile(file)
+        well = BaseWellFile(file)
         if search_criteria == "Well Name" and well.get_well_name() == criteria_value:
             plate_recording_list.append(file)
         if search_criteria == "Plate Barcode" and well.get_plate_barcode() == criteria_value:
@@ -174,7 +179,7 @@ def _extract_datetime_from_h5(
     )
 
 
-class BasicWellFile:
+class H5Wrapper:
     """Very thin wrapper around an H5 file for a single well of data.
 
     Used typically just for assessing file version when migrating.
@@ -293,13 +298,12 @@ class LikelyConsistentMetadata(
     """A wrapper for mixins that will likely be in all future file formats."""
 
 
-class WellFile(
-    BasicWellFile, LikelyConsistentMetadata
-):  # pylint: disable=too-many-ancestors # Eli (7/28/20): I don't see a way around this...we need to subclass h5py File
+class BaseWellFile(
+    H5Wrapper, LikelyConsistentMetadata
+):  # pylint: disable=too-many-ancestors # Tanner (6/9/21): consider refactoring the metadata mixins
     """Wrapper around an H5 file for a single well of data.
 
-    This is only guaranteed to function correctly on the current working file format version.
-    Use the file migrate_to_latest_version to get files up to date with the current working version.
+    abstract class containing the most generic functionality of Well Files.
 
     Args:
         file_name: The path of the H5 file to open.
@@ -308,11 +312,6 @@ class WellFile(
         _h5_file: The opened H5 file object.
     """
 
-    def __init__(self, file_name: str) -> None:
-        super().__init__(file_name)
-        self._raw_tissue_reading: Optional[NDArray[(Any, Any), int]] = None
-        self._raw_ref_reading: Optional[NDArray[(Any, Any), int]] = None
-
     def get_unique_recording_key(self) -> Tuple[str, datetime.datetime]:
         barcode = self.get_plate_barcode()
         start_time = self.get_begin_recording()
@@ -324,19 +323,13 @@ class WellFile(
         )
 
     def get_begin_recording(self) -> datetime.datetime:
-        return _extract_datetime_from_h5(
-            self._h5_file, self._file_version, UTC_BEGINNING_RECORDING_UUID
-        )
+        return _extract_datetime_from_h5(self._h5_file, self._file_version, UTC_BEGINNING_RECORDING_UUID)
 
     def get_timestamp_of_first_tissue_data_point(self) -> datetime.datetime:
-        return _extract_datetime_from_h5(
-            self._h5_file, self._file_version, UTC_FIRST_TISSUE_DATA_POINT_UUID
-        )
+        return _extract_datetime_from_h5(self._h5_file, self._file_version, UTC_FIRST_TISSUE_DATA_POINT_UUID)
 
     def get_timestamp_of_first_ref_data_point(self) -> datetime.datetime:
-        return _extract_datetime_from_h5(
-            self._h5_file, self._file_version, UTC_FIRST_REF_DATA_POINT_UUID
-        )
+        return _extract_datetime_from_h5(self._h5_file, self._file_version, UTC_FIRST_REF_DATA_POINT_UUID)
 
     def get_tissue_sampling_period_microseconds(self) -> int:
         return int(
@@ -371,7 +364,23 @@ class WellFile(
             )
         )
 
-    def get_raw_tissue_reading(self) -> NDArray[(Any, Any), int]:
+
+class Beta1WellFile(
+    BaseWellFile
+):  # pylint: disable=too-many-ancestors # Tanner (6/9/21): consider refactoring the metadata mixins
+    """Wrapper around H5 file of latest Beta 1 version.
+
+    This is only guaranteed to function correctly on the latest Beta 1
+    file format version. Use the file migrate_to_latest_version to get
+    files up to date with the current working version.
+    """
+
+    def __init__(self, file_name: str) -> None:
+        super().__init__(file_name)
+        self._raw_tissue_reading: Optional[NDArray[(2, Any), int]] = None
+        self._raw_ref_reading: Optional[NDArray[(2, Any), int]] = None
+
+    def get_raw_tissue_reading(self) -> NDArray[(2, Any), int]:
         """Get a value vs time array.
 
         Time (centi-milliseconds) is first dimension, value is second
@@ -383,7 +392,7 @@ class WellFile(
             self._raw_tissue_reading = self._load_reading(TISSUE_SENSOR_READINGS)
         return self._raw_tissue_reading
 
-    def get_raw_reference_reading(self) -> NDArray[(Any, Any), int]:
+    def get_raw_reference_reading(self) -> NDArray[(2, Any), int]:
         """Get a reference value vs time array.
 
         Time (centi-milliseconds) is first dimension, reference value is second
@@ -395,15 +404,12 @@ class WellFile(
             self._raw_ref_reading = self._load_reading(REFERENCE_SENSOR_READINGS)
         return self._raw_ref_reading
 
-    def _load_reading(self, reading_type: str) -> NDArray[(Any, Any), int]:
+    def _load_reading(self, reading_type: str) -> NDArray[(2, Any), int]:
         if reading_type not in (TISSUE_SENSOR_READINGS, REFERENCE_SENSOR_READINGS):
             raise NotImplementedError(reading_type)
-        recording_start_index_useconds = (
-            self.get_recording_start_index() * MICROSECONDS_PER_CENTIMILLISECOND
-        )
-        timestamp_of_start_index = (
-            self.get_timestamp_of_beginning_of_data_acquisition()
-            + datetime.timedelta(microseconds=recording_start_index_useconds)
+        recording_start_index_useconds = self.get_recording_start_index() * MICROSECONDS_PER_CENTIMILLISECOND
+        timestamp_of_start_index = self.get_timestamp_of_beginning_of_data_acquisition() + datetime.timedelta(
+            microseconds=recording_start_index_useconds
         )
         initial_timestamp = (
             self.get_timestamp_of_first_tissue_data_point()
@@ -422,20 +428,11 @@ class WellFile(
         )
         time_step = int(sampling_period / MICROSECONDS_PER_CENTIMILLISECOND)
 
-        # Tanner (5/6/21): adding `[:]` loads the data as a numpy array giving us more flexibility of multi-dimensional arrays
-        data = self._h5_file[reading_type][:]
-        if len(data.shape) == 1:
-            data = data.reshape(1, data.shape[0])
-        # fmt: off
-        # Tanner (5/6/21): black reformatted this into a very ugly few lines of code
-        times = np.mgrid[: data.shape[1],] * time_step
-        # fmt: on
-        time_delta_centimilliseconds = self._check_for_trimmed_file(
-            times[0], time_delta_centimilliseconds
-        )
-        return np.concatenate(  # pylint: disable=unexpected-keyword-arg # Tanner (5/6/21): unsure why pylint thinks dtype is an unexpected kwarg for np.concatenate
-            (times + time_delta_centimilliseconds, data), dtype=np.int32
-        )
+        data = self._h5_file[reading_type]
+        num_data_points = len(data)
+        times = np.arange(0, num_data_points * time_step, time_step)
+        time_delta_centimilliseconds = self._check_for_trimmed_file(times, time_delta_centimilliseconds)
+        return np.array((times + time_delta_centimilliseconds, data), dtype=np.int32)
 
     def _check_for_trimmed_file(
         self, times: NDArray[(1, Any), int], time_delta_centimilliseconds: int
@@ -454,6 +451,76 @@ class WellFile(
         return new_time_delta
 
 
+class WellFile(
+    BaseWellFile
+):  # pylint: disable=too-many-ancestors # Tanner (6/9/21): consider refactoring the metadata mixins
+    """Wrapper around H5 file of latest file format version.
+
+    This is only guaranteed to function correctly on the latest file
+    format version. Use the file migrate_to_latest_version to get files
+    up to date with the current working version.
+    """
+
+    def __init__(self, file_name: str) -> None:
+        super().__init__(file_name)
+        self._sensor_axis_dict: Dict[str, List[str]] = json.loads(
+            _get_file_attr(
+                self._h5_file,
+                str(MAGNETOMETER_CONFIGURATION_UUID),
+                self._file_version,
+            )
+        )
+        self._sensor_time_indices: Dict[str, NDArray[(1, Any), int]] = {
+            sensor: None for sensor in self._sensor_axis_dict.keys()
+        }
+        self._raw_channel_readings: Dict[str, Dict[str, NDArray[(1, Any), int]]] = {
+            sensor: {axis: None for axis in axes} for sensor, axes in self._sensor_axis_dict.items()
+        }
+
+    def get_magnetometer_config(self) -> Dict[str, List[str]]:
+        return self._sensor_axis_dict
+
+    def get_raw_time_indices(self) -> NDArray[(1, Any), int]:
+        return self._h5_file[TIME_INDICES]
+
+    def get_raw_channel_reading(self, sensor: str, axis: str) -> NDArray[(2, Any), int]:
+        """Get a value vs. time array of a single axis on a single sensor.
+
+        All values in array are int64
+        """
+        try:
+            if self._sensor_time_indices[sensor] is None:
+                sensor_time_offset_idx = list(self._sensor_axis_dict.keys()).index(sensor)
+                self._sensor_time_indices[sensor] = (
+                    self.get_raw_time_indices() - self._h5_file[TIME_OFFSETS][sensor_time_offset_idx, :]
+                )
+        except KeyError as e:
+            raise SensorDataNotInFileError(sensor) from e
+
+        try:
+            if self._raw_channel_readings[sensor][axis] is None:
+                channel_idx = self._get_channel_idx(sensor, axis)
+                self._raw_channel_readings[sensor][axis] = self._h5_file[TISSUE_SENSOR_READINGS][
+                    channel_idx, :
+                ]
+        except KeyError as e:
+            raise AxisDataForSensorNotInFileError(sensor, axis) from e
+
+        return np.array(
+            [self._sensor_time_indices[sensor], self._raw_channel_readings[sensor][axis]],
+            dtype=np.int64,
+        )
+
+    def _get_channel_idx(self, sensor: str, axis: str) -> int:
+        idx = 0
+        for sensor_name, axes in self._sensor_axis_dict.items():
+            if sensor_name == sensor:
+                return idx + axes.index(axis)
+            idx += len(axes)
+        # reaching this point this means the sensor wasn't found, but should be covered by try/except in get_raw_channel_reading
+        raise NotImplementedError("Sensor not found in data. This should be covered by prior checks")
+
+
 def find_start_index(from_start: int, old_data: NDArray[(1, Any), int]) -> int:
     start_index = 0
     time_from_start = 0
@@ -464,42 +531,6 @@ def find_start_index(from_start: int, old_data: NDArray[(1, Any), int]) -> int:
     return start_index - 1
 
 
-class WellFile_0_3_1(  # pylint:disable=invalid-name,too-many-ancestors # Eli (1/18/21): this seems like a good way to specifically name these historical class objects. I don't see a way around this ancestor issue...we need to subclass h5py File
-    WellFile
-):
-    """Historical class to open WellFiles in version 0.3.1.
-
-    Typically kept around for test cases and migration scripts.
-
-    If the main WellFile object stops being compatible, then old
-    deprecated methods should be moved here.
-    """
-
-
-class WellFile_0_4_1(  # pylint:disable=invalid-name,too-many-ancestors # Eli (1/18/21): this seems like a good way to specifically name these historical class objects. I don't see a way around this ancestor issue...we need to subclass h5py File
-    WellFile
-):
-    """Historical class to open WellFiles in version 0.4.1.
-
-    Typically kept around for test cases and migration scripts.
-
-    If the main WellFile object stops being compatible, then old
-    deprecated methods should be moved here.
-    """
-
-
-class WellFile_0_4_2(  # pylint:disable=invalid-name,too-many-ancestors # Eli (1/18/21): this seems like a good way to specifically name these historical class objects. I don't see a way around this ancestor issue...we need to subclass h5py File
-    WellFile
-):
-    """Historical class to open WellFiles in version 0.4.2.
-
-    Typically kept around for test cases and migration scripts.
-
-    If the main WellFile object stops being compatible, then old
-    deprecated methods should be moved here.
-    """
-
-
 class PlateRecording:
     """Wrapper around 24 WellFiles for a single plate of data.
 
@@ -510,15 +541,15 @@ class PlateRecording:
         _files : WellFiles of all the file paths provided.
     """
 
-    def __init__(self, file_paths: Sequence[Union[str, WellFile]]) -> None:
-        self._files: List[WellFile] = list()
-        self._wells_by_index: Dict[int, WellFile] = dict()
+    def __init__(self, file_paths: Sequence[Union[str, BaseWellFile]]) -> None:
+        self._files: List[BaseWellFile] = list()
+        self._wells_by_index: Dict[int, BaseWellFile] = dict()
         min_supported_version = VersionInfo.parse(MIN_SUPPORTED_FILE_VERSION)
         for iter_file_path in file_paths:
 
             well_file = iter_file_path
             if isinstance(well_file, str):
-                well_file = WellFile(well_file)
+                well_file = BaseWellFile(well_file)
             file_version_str = well_file.get_file_version()
             if file_version_str.split(".") < min_supported_version:
                 raise UnsupportedMantarrayFileVersionError(file_version_str)
@@ -535,7 +566,7 @@ class PlateRecording:
     def from_directory(cls, dir_to_load_files_from: str) -> "PlateRecording":
         return cls(glob(os.path.join(dir_to_load_files_from, "*.h5")))
 
-    def get_well_by_index(self, well_index: int) -> WellFile:
+    def get_well_by_index(self, well_index: int) -> BaseWellFile:
         return self._wells_by_index[well_index]
 
     def get_wellfile_names(self) -> Sequence[str]:
@@ -552,8 +583,3 @@ class PlateRecording:
 
     def get_well_indices(self) -> Tuple[int, ...]:
         return tuple(sorted(self._wells_by_index.keys()))
-
-
-WELL_FILE_CLASSES = immutabledict(
-    {"0.3.1": WellFile_0_3_1, "0.4.1": WellFile_0_4_1, "0.4.2": WellFile_0_4_2}
-)
